/*****************************************************************************************************
 *
 * CHECK_MRTG.C
 *
 * Program: MRTG (Multi-Router Traffic Grapher) generic plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-21-1999
 *
 * Command line: CHECK_MRTG <log_file> <expire_minutes> <AVG|MAX> <variable> <vwl> <vcl> <label> [rate]
 *
 * Description:
 *
 * This plugin will check either the average or maximum value of one of the two variables recorded
 * in an MRTG log file.  If the value exceeds the <vwl> threshold, a WARNING status is returned.
 * If the value exceeds the <vcl> threshold, a CRITICAL status is returned.  If the data in the log
 * file is older than <expire_minutes> old, a WARNING status is returned and a warning message is
 * printed.  
 *
 * This plugin is useful for monitoring MRTG data that does not correspond to bandwidth usage.
 * (Use the check_mrtgtraf plugin for monitoring bandwidth).  It can be used to monitor any kind of
 * data that MRTG is monitoring - errors, packets/sec, etc.  I use MRTG in conjuction with the Novell
 * NLM that allows me to track processor utilization, user connections, drive space, etc and this
 * plugin works well for monitoring that kind of data as well.
 *
 * Notes:
 * - This plugin only monitors one of the two variables stored in the MRTG log file.  If you want to
 *   monitor both values you'll have to define two commands with different values for the <variable>
 *   argument.  Of course, you can always hack the code to make this plugin work for you...
 * - MRTG stands for the Multi Router Traffic Grapher.  It can be downloaded from
 *   http://ee-staff.ethz.ch/~oetiker/webtools/mrtg/mrtg.html
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	1024


#define	TRUE		1
#define FALSE		0


int main(int argc, char **argv){
	int result;
	FILE *fp;
	int line;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	time_t current_time;
	int expire_minutes=0;
	int use_average=TRUE;
	int variable_number=1;
	unsigned long value_warning_threshold=0L;
	unsigned long value_critical_threshold=0L;
	char error_message[MAX_INPUT_BUFFER];
	char log_file[MAX_INPUT_BUFFER];
	time_t timestamp=0L;
	unsigned long average_value_rate=0L;
	unsigned long maximum_value_rate=0L;
	unsigned long value_rate=0L;
	char value_label[64];
	char value_rate_label[64];



	if(argc<8||argc>9){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("MRTG generic plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-21-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <log_file> <expire_minutes> <AVG | MAX> <variable> <vwl> <vcl> <label> [rate]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <log_file>       = The MRTG log file containing the data you want to monitor\n");
		printf(" <expire_minutes> = Minutes before MRTG data is considered to be too old\n");
		printf(" <AVG | MAX>	  = Should we check average or maximum values?\n");
		printf(" <variable>       = Which variable set should we inspect? 1 or 2?\n");
		printf(" <vwl>            = Threshold value for data to result in WARNING status\n");
		printf(" <vcl>            = Threshold value for data to result in CRITICAL status\n");
		printf(" <label>	  = Type label for data.\n");
		printf("                    Examples: Conns, \"Processor Load\", In, Out\n");
		printf(" [rate]           = Option rate label for data.\n");
		printf("                    Example: Packets/Sec, Errors/Sec, \"Bytes Per Second\", \"%% Utilization\"\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the MRTG log file name */
	strcpy(log_file,argv[1]);

	/* get the number of minutes before MRTG data "expires" and generates a WARNING status */
	expire_minutes=atoi(argv[2]);

	/* see if we should use max values instead of average values */
	if(!strcmp(argv[3],"MAX"))
		use_average=FALSE;
	else
		use_average=TRUE;

	/* get the variable number we should look at */
	variable_number=atoi(argv[4]);
	if(variable_number<1||variable_number>2){
		printf("Invalid variable number\n");
		return STATE_UNKNOWN;
	        }
	
	/* get the warning threshold level */
	value_warning_threshold=strtoul(argv[5],NULL,10);

	/* get the critical threshold level */
	value_critical_threshold=strtoul(argv[6],NULL,10);

	/* get the variable type label */
	strcpy(value_label,argv[7]);

	/* get the variable rate label */
	if(argc==9)
		strcpy(value_rate_label,argv[8]);
	else
		strcpy(value_rate_label,"");


	/* we haven't encountered any errors thus far... */
	result=STATE_OK;

	/* open the MRTG log file for reading */
	fp=fopen(log_file,"r");
	if(fp==NULL){
		printf("Unable to open MRTG log file\n");
		return STATE_UNKNOWN;
	        }

	line=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		line++;

		/* skip the first line of the log file */
		if(line==1)
			continue;

		/* break out of read loop if we've passed the number of entries we want to read */
		if(line>2)
			break;

		/* grab the timestamp */
		temp_buffer=strtok(input_buffer," ");
		timestamp=strtoul(temp_buffer,NULL,10);

		/* grab the average value 1 rate */
		temp_buffer=strtok(NULL," ");
		if(variable_number==1)
			average_value_rate=strtoul(temp_buffer,NULL,10);

		/* grab the average value 2 rate */
		temp_buffer=strtok(NULL," ");
		if(variable_number==2)
			average_value_rate=strtoul(temp_buffer,NULL,10);

		/* grab the maximum value 1 rate */
		temp_buffer=strtok(NULL," ");
		if(variable_number==1)
			maximum_value_rate=strtoul(temp_buffer,NULL,10);

		/* grab the maximum value 2 rate */
		temp_buffer=strtok(NULL," ");
		if(variable_number==2)
			maximum_value_rate=strtoul(temp_buffer,NULL,10);
	        }

	/* close the log file */
	fclose(fp);

	/* if we couldn't read enough data, return an unknown error */
	if(line<=2){
		result=STATE_UNKNOWN;
		sprintf(error_message,"Unable to process MRTG log file\n");
	        }

	/* make sure the MRTG data isn't too old */
	if(result==STATE_OK){
		time(&current_time);
		if(current_time-timestamp>(expire_minutes*60)){
			result=STATE_WARNING;
			sprintf(error_message,"MRTG data has expired (%d minutes old)\n",(int)((current_time-timestamp)/60));
		        }
	        }

	/* else check the incoming/outgoing rates */
	if(result==STATE_OK){

		if(use_average==TRUE)
			value_rate=average_value_rate;
		else
			value_rate=maximum_value_rate;

		if(value_rate>value_critical_threshold)
			result=STATE_CRITICAL;
		else if(value_rate>value_warning_threshold)
			result=STATE_WARNING;
	        }

	sprintf(error_message,"%s. %s = %lu %s",(use_average==TRUE)?"Ave":"Max",value_label,value_rate,value_rate_label);
	printf("%s\n",error_message);

	return result;
        }






