/*************************************************************
 *
 * Program: check_pgsql.c
 * License: GPL
 *
 * Written by Karl DeBisschop
 *  (based on plugins by Ethan Galstad and PostgresQL example code)
 * Last Modified: 19990422
 *
 * Command line: check_pgsql <warn_threshold> <host> <port> <database>
 *
 * Description:
 * This plugin will attempt to connect to the specified database
 * on the host.  Successul connects return STATE_OK, refusals
 * and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.
 **************************************************************/

#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <netdb.h>
#include <time.h>
#include <libpq-fe.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

time_t start_time,end_time;

int main(int argc, char **argv) {
  char *pghost;
  char *pgport;
  char *pgoptions;
  char *pgtty;
  char *dbName;
  char *user;
  char *passwd;
  char *textarg;
  int elapsed_time;
  int twarn;
  int tcrit;
  PGconn     *conn;
  /*  PGresult   *res; */

  if(argc>8){
    printf("Incorrect number of arguments supplied\n\n");
    printf("Check_pgsql plugin for NetSaint\n");
    printf("Copyright (c) 1999 Karl DeBisschop\n\n");
    printf("Usage: %s <twarn> <tcrit> <host> <port> <db> <user> <passwd>\n",argv[0]);
    printf("   <twarn>  = connection time to trigger warning status\n");
    printf("   <tcrit>  = connection time to trigger critical status\n");
    printf("   <host>   = name or ip address of machine running backend\n");
    printf("   <port>   = port running backend (5432 is default)\n");
    printf("   <db>     = name of database puging should connect to\n");
    printf("   <user>   = name of user\n");
    printf("   <passwd> = password for user (SECURITY ISSUES HERE)\n\n");
    printf("All parameters are optional.  The default is equivalent to:\n\n");
    printf("   %s 120 3600 localhost 5432\n\n",argv[0]);
    printf("and assumes that the user that runs the plugin (typically \n");
    printf("www or webadmin) can connect to the database without a password.\n");
    printf("This requires that the backend for remote machines uses TCP/IP\n");
    printf("(start postmaster with the -i noption)\n\n\n");
    return STATE_UNKNOWN;
  }

  /* begin, by setting the parameters for a backend connection if the
   * parameters are null, then the system will try to use reasonable
   * defaults by looking up environment variables or, failing that,
   * using hardwired constants */

  if (argc > 1) {
    textarg = argv[1];
    sscanf(textarg, "%d", &twarn);
  } else {
    twarn = 120;       /* host name of the backend server */
  }

  if (argc > 2) {
    textarg = argv[2];
    sscanf(textarg, "%d", &tcrit);
  } else {
    tcrit = 3600;
  }

  if (argc > 3) {
    pghost = argv[3];
  } else {
    pghost = NULL;       /* host name of the backend server */
  }

  if (argc > 4) {
    pgport = argv[4];
  } else {
    pgport = NULL;       /* port of the backend server */
  }

  if (argc > 5) {
    dbName = argv[5];
  } else {
    dbName = "template1";
  }

  if (argc > 6) {
    user = argv[6];
  } else {
    user = NULL;
  }

  if (argc > 7) {
    passwd = argv[7];
  } else {
    passwd = NULL;
  }

  pgoptions = NULL;    /* special options to start up the backend server */
  pgtty = NULL;        /* debugging tty for the backend server */

  /* make a connection to the database */
  time(&start_time);
  if (argc > 6) {
    conn = PQsetdbLogin(pghost, pgport, pgoptions, pgtty, dbName, user, passwd);
  } else {
    conn = PQsetdb(pghost, pgport, pgoptions, pgtty, dbName);
  }
  time(&end_time);
  elapsed_time = (int)(end_time - start_time);

  /* check to see that the backend connection was successfully made */
  if (PQstatus(conn) == CONNECTION_BAD) {
    printf("'%s' connection failed (%s).\n", dbName,PQerrorMessage(conn));
    PQfinish(conn);
    return STATE_CRITICAL;
  } else if (elapsed_time > tcrit) {
    PQfinish(conn);
    printf("Critcally slow connecting to postgresQL database %s (%d sec.)\n",dbName,elapsed_time);
    return STATE_CRITICAL;
  } else if (elapsed_time > twarn) {
    PQfinish(conn);
    printf("Slow connecting to postgresQL database %s (%d sec.)\n",dbName,elapsed_time);
    return STATE_WARNING;
  } else {
    PQfinish(conn);
    printf("Connected to postgresQL database %s (%d sec.)\n",dbName,elapsed_time);
    return STATE_OK;
  }
}
