/******************************************************************************************
 *
 * CHECK_PROCS.C
 *
 * Program: Process plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-22-1999
 *
 * Command line: CHECK_PROCS <wprocs> <cprocs> [proc_flags]
 *
 * Description:
 *
 * This plugin will check the number of currently running processes and generate WARNING
 * or CRITICAL states if the total number is above either the <wprocs> or <cprocs> values,
 * respectively.  Arguments in the [proc_flags] field should be one or more of the 
 * following: R (Running), S (Sleeping), T (Stopped or Traced), D (Uninterruptible Sleep),
 * Z (Zombie State).  Multiple flags can be used.  Example: 'RSTZD' (without quotes) will
 * count processes of all types,  'Z' will count only zombie processes, 'SD' will count
 * processes that are either sleeping or in an uninterruptible sleep.
 * The plugin gets process information by reading the output from the /bin/ps command.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256

int main(int argc, char **argv){
	int wprocs;
	int cprocs;
	int procs=0;
	int result=STATE_UNKNOWN;
	int found;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char proc_flags[8];
	char proc_status[8];
	char output_format[MAX_INPUT_BUFFER];

	if(argc<3||argc>4){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Process plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-22-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wprocs> <cprocs> [proc_flags]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wprocs>     = Number of processes necessary to result in a WARNING state\n");
		printf(" <cprocs>     = Number of processes necessary to result in a CRITICAL state\n");
		printf(" [proc_flags] = Only search for processes with one or more of these status\n");
		printf("                flags.  Status flags should be in the format that the 'ps'\n");
		printf("                command displays.  The plugin will examine the status field\n");
		printf("		from the output of the 'ps' command and count only those\n");
		printf("		processes which have a status flag found in the [proc_flags]\n");
		printf("		argument you supply.  Default is to count all types of procs.\n");
		printf("		Examples: R, Z, S, D, T, RS, RSZDT, etc.\n");
		printf("\n");
		printf("This plugin will check the number of processes on the local host and return an\n");
		printf("error if the number is above either of the thresholds given.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the number of procs use as warning and critical thresholds */
	wprocs=atoi(argv[1]);
	cprocs=atoi(argv[2]);

	/* get the status flags */
	if(argc==4){
		strcpy(proc_flags,argv[3]);
		sprintf(output_format,"%%s - %%d processes with %s status\n",argv[3]);
	        }
	else
		sprintf(output_format,"%%s - %%d processes running\n");

	/* run the command */
	fp=popen(PS_COMMAND,"r");
	if(fp==NULL){
		printf("Could not open pipe: %s\n",PS_COMMAND);
		return STATE_UNKNOWN;
	        }

	found=0;
	procs=0;

	fgets(input_buffer,MAX_INPUT_BUFFER-1,fp);

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		if(sscanf(input_buffer, PS_FORMAT, proc_status)){
			found++;
			if((argc==3) || strstr(proc_flags,proc_status))
				procs++;
		        }
		else
			printf("%s\n",input_buffer);
	        }

	/* close the temp file */
	pclose(fp);

	/* if we couldn't find the process count, return an UNKNOWN state */
	if(!found){
		printf("Unable to read output\n");
		return STATE_UNKNOWN;
	        }
	else if(procs<wprocs){
		printf(output_format,"OK",procs);
		return STATE_OK;
	        }
	else if(procs>=cprocs){
		printf(output_format,"CRITICAL",procs);
		return STATE_CRITICAL;
	        }
	else if(procs>=wprocs){
		printf(output_format,"WARNING",procs);
		return STATE_WARNING;
	        }

	return result;
        }
