/******************************************************************************************
 *
 * CHECK_VSZ.C
 *
 * Program: Process plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *
 * Last Modified: 18 May 1999
 *
 * Command line: check_vsz <wsize> <csize> [command_name]
 *
 * Description:
 *
 * This plugin will check for processes whose total image size exceeds the waring or
 * critical thresholds given on the command line.  With no command_name, everything that
 * shows up on ps is evaluated.  Otherwise, only jobs with the command_name given are
 * examined.  This program is particularly useful if you have to run a piece of commercial
 * software that has a memory leak.  With it you can shut down ad restart the processes
 * whenever the program threatens to take over your system.
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256

#define MYMAX(x,y) ((x)>(y) ? (x) : (y))

int main(int argc, char **argv){
	int wprocs;
	int cprocs;
	int proc_size;
	int result=STATE_OK;
	int line;
	int found;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char proc_name[16];
	char proc_to_find[16];
	char message[1024];

	if(argc<3||argc>4){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Process plugin for NetSaint\n");
		printf("Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)\n");
		printf("Last Modified: 18 May 1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wsize> <csize> [command]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wsize>     = Program image size necessary to cause a WARNING state\n");
		printf(" <csize>     = Program image size necessary to cause a CRITICAL state\n");
		printf(" [command]   = (optional) Program to search for\n");
		printf("\n");
		printf("This plugin will check image size of a running program and return an\n");
		printf("error if the number is above either of the thresholds given.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the number of procs use as warning and critical thresholds */
	wprocs=atoi(argv[1]);
	cprocs=atoi(argv[2]);

	/* get the status flags */
	if(argc==4)
		strcpy(proc_to_find,argv[3]);

	/* run the command */
	fp=popen(VSZ_COMMAND,"r");
	if(fp==NULL){
	  printf("Unable to open pipe: %s\n",VSZ_COMMAND);
	  return STATE_UNKNOWN;
	}

	found=0;
	line=0;
	proc_size=0;
	sprintf(message,"%s","");
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

	  line++;

	  /* skip the first line */
	  if(line==1)
	    continue;
	  sscanf(input_buffer,"%*s %*s %*s %*s %d %*s %*s %*s %*s %*s %s",&proc_size,proc_name);
	  if(proc_name!=NULL){
	    if(argc==3){
	      if(proc_size > wprocs){
		sprintf(message,"%s %s(%d)",message,proc_name,proc_size);
		result=MYMAX(result,STATE_WARNING);
	      }
	      if(proc_size > cprocs){
		result=STATE_CRITICAL;
	      }
	    }
	    else if(strstr(proc_name,proc_to_find)){
	      sprintf(message,"%s %d",message,proc_size);
	      if(proc_size > wprocs){
		result=MYMAX(result,STATE_WARNING);
	      }
	      if(proc_size > cprocs){
		result=STATE_CRITICAL;
	      }
	    }
	  } 
	}

	/* close the temp file */
	fclose(fp);

	if(result==STATE_OK)
		printf("ok (all VSZ<%d): %s\n",wprocs,message);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n");
	else if(result==STATE_WARNING)
		printf("WARNING (VSZ>%d):%s\n",wprocs,message);
	else{
		printf("CRITICAL (VSZ>%d):%s\n",cprocs,message);
	        }

	return result;
        }
