/******************************************************************************************
 *
 * CHECK_UPS.C
 *
 * Program: UPS monitor plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-31-1999
 *
 * Command line: CHECK_UPS <host_address> [-u ups] [-p port] [-v variable] \
 *                         [-wv warn_value] [-cv crit_value] [-to to_sec]
 *
 * Description:
 *
 * This plugin attempts to determine the status of an UPS (Uninterruptible Power Supply)
 * on a remote host (or the local host) that is being monitored with Russel Kroll's
 * "Smarty UPS Tools" package.  If the UPS is online or calibrating, the plugin will
 * return an OK state.  If the battery is on it will return a WARNING state.  If the UPS
 * is off or has a low battery the plugin will return a CRITICAL state.  You may also
 * specify a variable to check (such as temperature, utility voltage, battery load, etc.)
 * as well as warning and critical thresholds for the value of that variable.  If the
 * remote host has multiple UPS that are being monitored you will have to use the [ups]
 * option to specify which UPS to check.
 *
 * Notes:
 *
 * This plugin requires that the UPSD daemon distributed with Russel Kroll's
 * "Smart UPS Tools" be installed on the remote host.  If you don't have the package
 * installed on your system, you can download it from 
 * http://www.exploits.org/~rkroll/smartupstools
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
*****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#else
#include <string.h>
#endif
#include <time.h>
#include <signal.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define CHECK_NONE	0


#define TRUE		1
#define FALSE		0

#define OK		0
#define ERROR		-1

#define UPS_PORT	3305
#define UPS_PROTO	"tcp"

#define DEFAULT_TIMEOUT		10	/* timeout after 10 seconds */

#define MAX_INPUT_BUFFER	256

#define UPS_NONE	0	/* no supported options */
#define UPS_UTILITY	1	/* supports utility line voltage */
#define UPS_BATTPCT	2	/* supports percent battery remaining */
#define UPS_STATUS	4	/* supports UPS status */
#define UPS_TEMP	8	/* supports UPS temperature */
#define UPS_LOADPCT	16	/* supports load percent */

time_t start_time,end_time;
int server_port=UPS_PORT;
char server_name[MAX_INPUT_BUFFER]="";
char ups_name[MAX_INPUT_BUFFER]="";
double warning_value=0.0L;
double critical_value=0.0L;
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int check_variable=UPS_NONE;
int timeout=DEFAULT_TIMEOUT;
int supported_options=UPS_NONE;

double ups_utility_voltage=0.0L;
double ups_battery_percent=0.0L;
double ups_load_percent=0.0L;
double ups_temperature=0.0L;
char ups_status[MAX_INPUT_BUFFER]="N/A";

int determine_supported_vars(void);
int get_ups_variable(char *,char *,int);
int my_connect(char *address,int port,char *protocol,int *sd);
int process_request(char *,int,char *,char *);
int process_arguments(int,char **);
void alarm_handler(int);


int main(int argc, char **argv){
	int result;
	int return_result;
	char output_message[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];

	double ups_utility_deviation=0.0L;

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("UPS monitor plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-26-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-u ups] [-v variable] [-wv warn_value] [-cv crit_value] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [port]       = Optional port number to use - default is %d\n",server_port);
		printf(" [ups]        = The name of the UPS to be checked if more than one UPS is being\n");
		printf("                monitored by the remote daemon.\n");
		printf(" [variable]   = Variable to check.  Valid variables include:\n");
		printf("                   LINE    = Utility line voltage (deviation from 120VAC)\n");
		printf("                   BATTPCT = Percent battery charge (0.0-100.0)\n");
		printf("                   LOADPCT = Percent load (0.0-100.0)\n");
		printf("                   TEMP    = Temperature (Degrees Farenheit)\n");
		printf(" [warn_value] = Threshold for variable value necessary to result in a warning status\n");
		printf(" [crit_value] = Threshold for variable value necessary to result in a critical status\n");
		printf(" [to_sec]     = Number of seconds before connection attempt times out - default is %d sec\n",DEFAULT_TIMEOUT);
		printf("\n");
		printf("This plugin attempts to contact the UPSD daemon running on the remote host to gather the\n");
		printf("requested UPS information.\n");
		printf("\n");
		printf("Notes:\n");
		printf("\n");
		printf("  - This plugin requires that the UPSD daemon distributed with Russell Kroll's\n");
		printf("    \"Smart UPS Tools\" be running on the remote host.  If you don't have the\n");
		printf("    package installed on your system, you can download it from\n");
		printf("    http://www.exploits.org/~kroll/smartupstools\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	return_result=STATE_OK;

	/* initialize alarm signal handling */
	signal(SIGALRM,alarm_handler);

	/* set socket timeout */
	alarm(timeout);

	/* determine what variables the UPS supports so we know what we can check */
	result=determine_supported_vars();
	if(result!=OK)
		return STATE_CRITICAL;

	/* get the ups status if possible */
	if(supported_options & UPS_STATUS){

		result=get_ups_variable("STATUS",temp_buffer,sizeof(temp_buffer));
		if(result!=OK)
			return STATE_CRITICAL;

		strcpy(ups_status,temp_buffer);
		ups_status[strlen(ups_status)-1]='\x0';

		if(!strcmp(ups_status,"OFF")){
			strcpy(ups_status,"Off");
			return_result=STATE_CRITICAL;
		        }
		else if(!strcmp(ups_status,"OL")){
			strcpy(ups_status,"Online");
			return_result=STATE_OK;
		        }
		else if(!strcmp(ups_status,"OB")){
			strcpy(ups_status,"On Battery");
			return_result=STATE_WARNING;
		        }
		else if(!strcmp(ups_status,"LB")){
			strcpy(ups_status,"Low Battery");
			return_result=STATE_CRITICAL;
		        }
		else if(!strcmp(ups_status,"CAL")){
			strcpy(ups_status,"Calibrating");
			return_result=STATE_OK;
		        }
		else{
			strcpy(ups_status,"Unknown");
			return_result=STATE_OK;
		        }
	        }

	/* get the ups utility voltage if possible */
	if(supported_options & UPS_UTILITY){

		result=get_ups_variable("UTILITY",temp_buffer,sizeof(temp_buffer));
		if(result!=OK)
			return STATE_CRITICAL;

		ups_utility_voltage=atof(temp_buffer);

		if(ups_utility_voltage>120.0)
			ups_utility_deviation=120.0-ups_utility_voltage;
		else
			ups_utility_deviation=ups_utility_voltage-120.0;

		if(check_variable==UPS_UTILITY){
			if(check_critical_value==TRUE && ups_utility_deviation >= critical_value)
				return_result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && ups_utility_deviation >= warning_value && return_result < STATE_WARNING)
				return_result=STATE_WARNING;
		        }
	        }

	/* get the ups battery percent if possible */
	if(supported_options & UPS_BATTPCT){

		result=get_ups_variable("BATTPCT",temp_buffer,sizeof(temp_buffer));
		if(result!=OK)
			return STATE_CRITICAL;

		ups_battery_percent=atof(temp_buffer);

		if(check_variable==UPS_BATTPCT){
			if(check_critical_value==TRUE && ups_battery_percent <= critical_value)
				return_result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && ups_battery_percent <= warning_value && return_result < STATE_WARNING)
				return_result=STATE_WARNING;
		        }
	        }

	/* get the ups load percent if possible */
	if(supported_options & UPS_LOADPCT){

		result=get_ups_variable("LOADPCT",temp_buffer,sizeof(temp_buffer));
		if(result!=OK)
			return STATE_CRITICAL;

		ups_load_percent=atof(temp_buffer);

		if(check_variable==UPS_LOADPCT){
			if(check_critical_value==TRUE && ups_load_percent >= critical_value)
				return_result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && ups_load_percent >= warning_value && return_result < STATE_WARNING)
				return_result=STATE_WARNING;
		        }
	        }

	/* get the ups temperature if possible */
	if(supported_options & UPS_TEMP){

		result=get_ups_variable("UPSTEMP",temp_buffer,sizeof(temp_buffer));
		if(result!=OK)
			return STATE_CRITICAL;

		ups_temperature=(atof(temp_buffer)*1.8)+32;

		if(check_variable==UPS_TEMP){
			if(check_critical_value==TRUE && ups_temperature >= critical_value)
				return_result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && ups_temperature >= warning_value && return_result < STATE_WARNING)
				return_result=STATE_WARNING;
		        }
	        }

	/* if the UPS does not support any options we are looking for, report an error */
	if(supported_options==UPS_NONE)
		return_result=STATE_CRITICAL;

	/* reset timeout */
	alarm(0);


        sprintf(output_message,"UPS %s - ",(return_result==STATE_OK)?"ok":"problem");

	if(supported_options & UPS_STATUS){
		sprintf(temp_buffer,"Status=%s ",ups_status);
		strcat(output_message,temp_buffer);
	        }
	if(supported_options & UPS_UTILITY){
		sprintf(temp_buffer,"Utility=%3.1lfV ",ups_utility_voltage);
		strcat(output_message,temp_buffer);
	        }
	if(supported_options & UPS_BATTPCT){
		sprintf(temp_buffer,"Batt=%3.1lf%% ",ups_battery_percent);
		strcat(output_message,temp_buffer);
	        }
	if(supported_options & UPS_LOADPCT){
		sprintf(temp_buffer,"Load=%3.1lf%% ",ups_load_percent);
		strcat(output_message,temp_buffer);
	        }
	if(supported_options & UPS_TEMP){
		sprintf(temp_buffer,"Temp=%3.1lfF",ups_temperature);
		strcat(output_message,temp_buffer);
	        }
	if(supported_options==UPS_NONE){
		sprintf(temp_buffer,"UPS does not appear to support any available options\n");
		strcat(output_message,temp_buffer);
	        }

	printf("%s\n",output_message);

	return return_result;
        }



/* determines what options are supported by the UPS */
int determine_supported_vars(void){
	char send_buffer[MAX_INPUT_BUFFER];
	char recv_buffer[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	int result;
	char *ptr;


	/* get the list of variables that this UPS supports */
	if(strcmp(ups_name,""))
		sprintf(send_buffer,"LISTVARS %s\r\n",ups_name);
	else
		sprintf(send_buffer,"LISTVARS\r\n");
	result=process_request(server_name,server_port,send_buffer,recv_buffer);
	if(result!=STATE_OK){
		printf("Invalid response received from host\n");
		return ERROR;
	        }

	recv_buffer[strlen(recv_buffer)-1]='\x0';

	if(strcmp(ups_name,""))
		ptr=recv_buffer+5+strlen(ups_name)+2;
	else
		ptr=recv_buffer+5;

	strcpy(temp_buffer,recv_buffer);

	for(ptr=strtok(temp_buffer," ");ptr!=NULL;ptr=strtok(NULL," ")){
		if(!strcmp(ptr,"UTILITY"))
			supported_options|=UPS_UTILITY;
		else if(!strcmp(ptr,"BATTPCT"))
			supported_options|=UPS_BATTPCT;
		else if(!strcmp(ptr,"LOADPCT"))
			supported_options|=UPS_LOADPCT;
		else if(!strcmp(ptr,"STATUS"))
			supported_options|=UPS_STATUS;
		else if(!strcmp(ptr,"UPSTEMP"))
			supported_options|=UPS_TEMP;
	        }

	return OK;
        }


/* gets a variable value for a specific UPS  */
int get_ups_variable(char *varname,char *buf,int buflen){
	char command[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char send_buffer[MAX_INPUT_BUFFER];
	char *ptr;
	int result;

	/* create the command string to send to the UPS daemon */
	if (!strcmp(ups_name,""))
		sprintf(send_buffer,"REQ %s\n",varname);
	else
		sprintf(send_buffer,"REQ %s@%s\n",varname,ups_name);

	/* send the command to the daemon and get a response back */
	result=process_request(server_name,server_port,send_buffer,temp_buffer);
	if(result!=STATE_OK){
		printf("Invalid response received from host\n");
		return ERROR;
	        }

	if(!strcmp(ups_name,""))
		ptr=temp_buffer+strlen(varname)+5;
	else
		ptr=temp_buffer+strlen(varname)+5+strlen(ups_name)+1;

	if(!strcmp(ptr,"NOT-SUPPORTED")){
		printf("Error: Variable '%s' is not supported\n",varname);
		return ERROR;
	        }

	if(!strcmp(ptr,"DATA-STALE")){
		printf("Error: UPS data is stale\n");
		return ERROR;
	        }

	if(!strcmp(ptr,"UNKNOWN-UPS")){
		printf("Error: UPS '%s' is unknown\n",ups_name);
		return ERROR;
	        }

	strcpy(buf,ptr);

	return OK;
        }


int process_arguments(int argc, char **argv){
	char buffer[MAX_INPUT_BUFFER];
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strcpy(server_name,argv[1]);

	for(x=3;x<=argc;x++){

		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-u")){
			if(x<argc){
				strcpy(ups_name,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-v")){
			if(x<argc){
				
				if(!strcmp(argv[x],"LINE"))
					check_variable=UPS_UTILITY;
				else if(!strcmp(argv[x],"TEMP"))
					check_variable=UPS_TEMP;
				else if(!strcmp(argv[x],"BATTPCT"))
					check_variable=UPS_BATTPCT;
				else if(!strcmp(argv[x],"LOADPCT"))
					check_variable=UPS_LOADPCT;
				else
					return ERROR;

				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wv")){
			if(x<argc){
				warning_value=strtoul(argv[x],NULL,10);
				check_warning_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-cv")){
			if(x<argc){
				critical_value=strtoul(argv[x],NULL,10);
				check_critical_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				timeout=atoi(argv[x]);
				if(timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }


int my_connect(char *host_name,int port,char *protocol,int *sd){
	struct sockaddr_in servaddr;
	struct hostent *hp;
	struct protoent *ptrp;
	int result;

	bzero((char *)&servaddr,sizeof(servaddr));
	servaddr.sin_family=AF_INET;
	servaddr.sin_port=htons(port);

	/* try to bypass using a DNS lookup if this is just an IP address */
	if(!inet_aton(host_name,&servaddr.sin_addr)){

		/* else do a DNS lookup */
		hp=gethostbyname((const char *)host_name);
		if(hp==NULL){
			printf("Invalid host name '%s'\n",host_name);
			return STATE_UNKNOWN;
		        }
    
		memcpy(&servaddr.sin_addr,hp->h_addr,hp->h_length);
	        }

	/* map TCP transport protocol name to protocol number */
	if(((int)(ptrp=getprotobyname(protocol)))==0){
		printf("Cannot map \"%s\" to protocol number\n",protocol);
		return STATE_UNKNOWN;
	        }

	/* create a socket */
	*sd=socket(PF_INET,SOCK_STREAM,ptrp->p_proto);
	if(*sd<0){
		printf("Socket creation failed\n");
		return STATE_UNKNOWN;
	        }

	/* open a connection */
	time(&start_time);
	result=connect(*sd,(struct sockaddr *)&servaddr,sizeof(servaddr));
	if(result<0){
		switch(errno){  
		case ECONNREFUSED:
			if(port==UPS_PORT)
				printf("Connection refused by host\n");
			else
				printf("Connection refused at port %d by host\n",port);
			break;
		case ETIMEDOUT:
			printf("Timeout while attempting connection\n");
			break;
		case ENETUNREACH:
			printf("Network is unreachable\n");
			break;
		default:
			printf("Connection refused or timed out\n");
		        }

		return STATE_CRITICAL;
	        }

	return STATE_OK;
        }



int process_request(char *server_name, int server_port, char *send_buffer, char *recv_buffer){
	int result;
	int send_result;
	int recv_result;
	int sd;
	char temp_buffer[MAX_INPUT_BUFFER];

	result=STATE_OK;

	/* try to connect to the UPS daemon on the specified host */
	result=my_connect(server_name,server_port,UPS_PROTO,&sd);
	if(result!=STATE_OK)
		return STATE_CRITICAL;

	send_result=send(sd,send_buffer,strlen(send_buffer),0);
	if(send_result!=strlen(send_buffer)){
		printf("send() failed\n");
		result=STATE_WARNING;
	        }

	recv_result=recv(sd,recv_buffer,MAX_INPUT_BUFFER-1,0);
	if(recv_result==-1){
		printf("recv() failed\n");
		result=STATE_WARNING;
	        }

	/* terminate returned string */
	recv_buffer[recv_result]='\x0';

	/* close the connection */
	strcpy(temp_buffer,"LOGOUT\n");
	send(sd,temp_buffer,strlen(send_buffer),0);
	close(sd);

	return result;
        }



void alarm_handler(int sig){

	printf("Socket timeout after %d seconds\n",timeout);

	exit(STATE_CRITICAL);
        }

